from pathlib import Path
from typing import Union
from instaui import ui
from instaui.dependencies.plugin_dependency import register_plugin
from instaui.arco.types import TLocale, TCustomizeLocale
from instaui.arco._settings import configure

static_folder = Path(__file__).parent / "static"

arco_css = static_folder / "instaui-arco.css"
arco_esm_js = static_folder / "instaui-arco.js"


def _register_arco():
    register_plugin("InstauiArco", esm=arco_esm_js, css=[arco_css])


def use(*, locale: ui.TMaybeRef[Union[TLocale, TCustomizeLocale]] = "en-US"):
    """Use arco ui.

    Args:
        locale (ui.TMaybeRef[Union[TLocale, TCustomizeLocale]], optional): The locale of arco ui. Defaults to "en-US".

    Examples:
    .. code-block:: python
        from instaui import ui, arco

        arco.use()

        @ui.page("/")
        def index_page():
            arco.input(placeholder="input")
    """

    _register_arco()
    configure(locale=locale)



from __future__ import annotations
import typing
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui.components.element import Element
from instaui.components.content import Content
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Button(Element):
    def __init__(
        self,
        label: typing.Optional[TMaybeRef[str]] = None,
        **kwargs: Unpack[component_types.TButton],
    ):
        """Create a button element.

        Args:
            text (Optional[TMaybeRef[str]], optional): _description_. Defaults to None.
        """

        super().__init__("a-button")

        if label is not None:
            with self:
                Content(label)

        self.props(handle_props(kwargs))  # type: ignore

    def on_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "click",
            handler,
            extends=extends,
        )
        return self



import typing
from typing_extensions import Unpack, TypedDict
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props, try_setup_vmodel


class TRadioGroup(TypedDict, total=False):
    model_value: typing.Union[str, int, bool]
    default_value: typing.Union[str, int, bool]
    type: typing.Literal["radio", "button"]
    size: typing.Literal["mini", "small", "medium", "large"]
    options: typing.List[typing.Union[str, int]]
    direction: typing.Literal["horizontal", "vertical"]
    disabled: bool


class RadioGroup(Element):
    def __init__(
        self,
        value: typing.Optional[TMaybeRef[typing.Union[str, float]]] = None,
        **kwargs: Unpack[TRadioGroup],
    ):
        super().__init__("a-radio-group")

        try_setup_vmodel(self, value)
        self.props(handle_props(kwargs))  # type: ignore

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self



from typing import Dict
from instaui.components.element import Element
from instaui.vars.mixin_types.element_binding import ElementBindingMixin


def handle_props(props: Dict):
    return {k.replace("_", "-"): v for k, v in props.items()}


def try_setup_vmodel(
    element: Element,
    value,
    *,
    prop_name: str = "value",
):
    if value is None:
        return
    if isinstance(value, ElementBindingMixin):
        element.vmodel(value, prop_name=prop_name)
        return

    element.props({prop_name: value})



from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.arco import component_types
from ._utils import handle_props


class Space(Element):
    def __init__(
        self,
        **kwargs: Unpack[component_types.TSpace],
    ):
        super().__init__("a-space")

        self.props(handle_props(kwargs))  # type: ignore



import typing
from typing_extensions import TypedDict, Unpack
from instaui.components.element import Element
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class TLayoutSider(TypedDict, total=False):
    theme: typing.Literal["dark", "light"]
    collapsed: bool
    default_collapsed: bool
    collapsible: bool
    width: int
    collapsed_width: int
    reverse_arrow: bool
    breakpoint: typing.Literal["xxl", "xl", "lg", "md", "sm", "xs"]
    resize_directions: typing.List[typing.Literal["left", "right", "top", "bottom"]]
    hide_trigger: bool


class LayoutSider(Element):
    def __init__(
        self,
        **kwargs: Unpack[TLayoutSider],
    ):
        super().__init__("a-layout-sider")
        self.props(handle_props(kwargs))  # type: ignore

    def on_collapse(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "collapse",
            handler,
            extends=extends,
        )
        return self

    def on_breakpoint(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "breakpoint",
            handler,
            extends=extends,
        )
        return self



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Alert(Element):
    def __init__(
        self,
        **kwargs: Unpack[component_types.TAlert],
    ):
        super().__init__("a-alert")

        self.props(handle_props(kwargs))  # type: ignore

    def on_close(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "close",
            handler,
            extends=extends,
        )
        return self

    def on_after_close(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "after-close",
            handler,
            extends=extends,
        )
        return self



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.components.content import Content
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props, try_setup_vmodel


class Menu(Element):
    def __init__(
        self,
        collapsed_value: typing.Optional[TMaybeRef[bool]] = None,
        **kwargs: Unpack[component_types.TMenu],
    ):
        super().__init__("a-menu")

        try_setup_vmodel(self, collapsed_value, prop_name="collapsed")
        self.props(handle_props(kwargs))  # type: ignore

    def on_collapse(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "collapse",
            handler,
            extends=extends,
        )
        return self

    def on_menu_item_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "menu-item-click",
            handler,
            extends=extends,
        )
        return self

    def on_sub_menu_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "sub-menu-click",
            handler,
            extends=extends,
        )
        return self


class MenuItem(Element):
    def __init__(self, title: typing.Optional[TMaybeRef[str]] = None):
        super().__init__("a-menu-item")

        if title is not None:
            with self:
                Content(title)

    def icon_slot(self):
        return self.add_slot("icon")


class SubMenu(Element):
    def __init__(
        self,
        title: typing.Optional[TMaybeRef[str]] = None,
    ):
        super().__init__("a-sub-menu")
        if title is not None:
            with self.title_slot():
                Content(title)

    def icon_slot(self):
        return self.add_slot("icon")

    def title_slot(self):
        return self.add_slot("title")



locale = {
    "locale": "en-US",
    "empty": {"description": "No Data"},
    "drawer": {"okText": "Ok", "cancelText": "Cancel"},
    "popconfirm": {"okText": "Ok", "cancelText": "Cancel"},
    "modal": {"okText": "Ok", "cancelText": "Cancel"},
    "pagination": {
        "goto": "Goto",
        "page": "Page",
        "countPerPage": " / Page",
        "total": "Total: {0}",
    },
    "table": {"okText": "Ok", "resetText": "Reset"},
    "upload": {
        "start": "Start",
        "cancel": "Cancel",
        "delete": "Delete",
        "retry": "Click to retry",
        "buttonText": "Upload",
        "preview": "Preview",
        "drag": "Click or drag file to this area to upload",
        "dragHover": "Release to upload",
        "error": "Upload Error",
    },
    "calendar": {
        "formatYear": "YYYY",
        "formatMonth": "MMM YYYY",
        "today": "Today",
        "view": {"month": "Month", "year": "Year", "week": "Week", "day": "Day"},
        "month": {
            "long": {
                "January": "January",
                "February": "February",
                "March": "March",
                "April": "April",
                "May": "May",
                "June": "June",
                "July": "July",
                "August": "August",
                "September": "September",
                "October": "October",
                "November": "November",
                "December": "December",
            },
            "short": {
                "January": "Jan",
                "February": "Feb",
                "March": "Mar",
                "April": "Apr",
                "May": "May",
                "June": "Jun",
                "July": "Jul",
                "August": "Aug",
                "September": "Sept",
                "October": "Oct",
                "November": "Nov",
                "December": "Dec",
            },
        },
        "week": {
            "long": {
                "self": "Week",
                "monday": "Monday",
                "tuesday": "Tuesday",
                "wednesday": "Wednesday",
                "thursday": "Thursday",
                "friday": "Friday",
                "saturday": "Saturday",
                "sunday": "Sunday",
            },
            "short": {
                "self": "Week",
                "monday": "Mon",
                "tuesday": "Tue",
                "wednesday": "Wed",
                "thursday": "Thu",
                "friday": "Fri",
                "saturday": "Sat",
                "sunday": "Sun",
            },
        },
    },
    "datePicker": {
        "view": {"month": "Month", "year": "Year", "week": "Week", "day": "Day"},
        "month": {
            "long": {
                "January": "January",
                "February": "February",
                "March": "March",
                "April": "April",
                "May": "May",
                "June": "June",
                "July": "July",
                "August": "August",
                "September": "September",
                "October": "October",
                "November": "November",
                "December": "December",
            },
            "short": {
                "January": "Jan",
                "February": "Feb",
                "March": "Mar",
                "April": "Apr",
                "May": "May",
                "June": "Jun",
                "July": "Jul",
                "August": "Aug",
                "September": "Sept",
                "October": "Oct",
                "November": "Nov",
                "December": "Dec",
            },
        },
        "week": {
            "long": {
                "self": "Week",
                "monday": "Monday",
                "tuesday": "Tuesday",
                "wednesday": "Wednesday",
                "thursday": "Thursday",
                "friday": "Friday",
                "saturday": "Saturday",
                "sunday": "Sunday",
            },
            "short": {
                "self": "Week",
                "monday": "Mon",
                "tuesday": "Tue",
                "wednesday": "Wed",
                "thursday": "Thu",
                "friday": "Fri",
                "saturday": "Sat",
                "sunday": "Sun",
            },
        },
        "placeholder": {
            "date": "Please select date",
            "week": "Please select week",
            "month": "Please select month",
            "year": "Please select year",
            "quarter": "Please select quarter",
            "time": "Please select time",
        },
        "rangePlaceholder": {
            "date": ["Start date", "End date"],
            "week": ["Start week", "End week"],
            "month": ["Start month", "End month"],
            "year": ["Start year", "End year"],
            "quarter": ["Start quarter", "End quarter"],
            "time": ["Start time", "End time"],
        },
        "selectTime": "Select time",
        "today": "Today",
        "now": "Now",
        "ok": "Ok",
    },
    "image": {"loading": "loading"},
    "imagePreview": {
        "fullScreen": "Full Screen",
        "rotateRight": "Rotate Right",
        "rotateLeft": "Rotate Left",
        "zoomIn": "Zoom In",
        "zoomOut": "Zoom Out",
        "originalSize": "Original Size",
    },
    "typography": {
        "copied": "Copied",
        "copy": "Copy",
        "expand": "Expand",
        "collapse": "Collapse",
        "edit": "Edit",
    },
    "form": {
        "validateMessages": {
            "required": "#{field} is required",
            "type": {
                "ip": "#{field} is not a #{type} type",
                "email": "#{field} is not a #{type} type",
                "url": "#{field} is not a #{type} type",
                "string": "#{field} is not a #{type} type",
                "number": "#{field} is not a #{type} type",
                "array": "#{field} is not a #{type} type",
                "object": "#{field} is not a #{type} type",
                "boolean": "#{field} is not a #{type} type",
            },
            "number": {
                "min": "`#{value}` is not greater than `#{min}`",
                "max": "`#{value}` is not less than `#{max}`",
                "equal": "`#{value}` is not equal to `#{equal}`",
                "range": "`#{value}` is not in range `#{min} ~ #{max}`",
                "positive": "`#{value}` is not a positive number",
                "negative": "`#{value}` is not a negative number",
            },
            "string": {
                "maxLength": "#{field} cannot be longer than #{maxLength} characters",
                "minLength": "#{field} must be at least #{minLength} characters",
                "length": "#{field} must be exactly #{length} characters",
                "match": "`#{value}` does not match pattern #{pattern}",
                "uppercase": "`#{value}` must be all uppercase",
                "lowercase": "`#{value}` must be all lowercased",
            },
            "array": {
                "length": "#{field} must be exactly #{length} in length",
                "minLength": "#{field} cannot be less than #{minLength} in length",
                "maxLength": "#{field} cannot be greater than #{maxLength} in length",
                "includes": "#{field} is not includes #{includes}",
                "deepEqual": "#{field} is not deep equal with #{deepEqual}",
                "empty": "#{field} is not an empty array",
            },
            "object": {
                "deepEqual": "#{field} is not deep equal to expected value",
                "hasKeys": "#{field} does not contain required fields",
                "empty": "#{field} is not an empty object",
            },
            "boolean": {
                "true": "Expect true but got `#{value}`",
                "false": "Expect false but got `#{value}`",
            },
        }
    },
    "colorPicker": {
        "history": "History Colors",
        "preset": "Preset Colors",
        "empty": "Empty",
    },
}



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props, try_setup_vmodel


class Split(Element):
    def __init__(
        self,
        size_value: typing.Optional[TMaybeRef[typing.Union[int, float, str]]] = None,
        **kwargs: Unpack[component_types.TSplit],
    ):
        """Split element.


        Example:
        .. code-block:: python
            with arco.split(direction="vertical").props("max=0.8 min=0.2") as split:
                with split.first():
                    html.paragraph("first")
                with split.second():
                    html.paragraph("second")
        """

        super().__init__("a-split")

        try_setup_vmodel(self, size_value, prop_name="size")
        self.props(handle_props(kwargs))  # type: ignore

    def first(self):
        return self.add_slot("first")

    def second(self):
        return self.add_slot("second")

    def on_move_start(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "move-start",
            handler,
            extends=extends,
        )
        return self

    def on_moving(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "moving",
            handler,
            extends=extends,
        )
        return self

    def on_move_end(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "move-end",
            handler,
            extends=extends,
        )
        return self



from typing import Callable, Dict, Optional, Union, cast
from instaui import ui
from instaui.arco.types import TLocale, TCustomizeLocale
from instaui.arco.components.config_provider import ConfigProvider
from instaui.arco.locales import get_locale


_STOP_LIFESPAN: Optional[Callable] = None


def configure(*, locale: ui.TMaybeRef[Union[TLocale, TCustomizeLocale]]):
    global _STOP_LIFESPAN

    if isinstance(locale, str):
        locale = get_locale(locale)

    def add_config_provider_lifespan():
        with ConfigProvider(locale=cast(Dict, locale)):
            yield

    if _STOP_LIFESPAN is not None:
        _STOP_LIFESPAN()
        del _STOP_LIFESPAN

    _STOP_LIFESPAN = ui.on_page_request_lifespan(add_config_provider_lifespan)



"""
This file is automatically generated by InstaUI. Do not modify it manually.
"""

from typing import Union, Dict, List, Any, Literal, Tuple, Optional
import datetime
from typing_extensions import TypedDict
from instaui.vars.types import TMaybeRef


class TButton(TypedDict, total=False):
    type: TMaybeRef[
        Union[str, Literal["primary", "secondary", "dashed", "outline", "text"]]
    ]
    shape: TMaybeRef[Union[str, Literal["square", "round", "circle"]]]
    status: TMaybeRef[Union[str, Literal["normal", "warning", "success", "danger"]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    long: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    html_type: TMaybeRef[str]
    autofocus: TMaybeRef[bool]
    href: TMaybeRef[str]


class TLink(TypedDict, total=False):
    href: TMaybeRef[str]
    status: TMaybeRef[Union[str, Literal["normal", "warning", "success", "danger"]]]
    hoverable: TMaybeRef[bool]
    icon: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    disabled: TMaybeRef[bool]


class TTypography(TypedDict, total=False):
    type: TMaybeRef[
        Union[str, Literal["primary", "secondary", "success", "danger", "warning"]]
    ]
    bold: TMaybeRef[bool]
    mark: TMaybeRef[Union[bool, Dict[str, str]]]
    underline: TMaybeRef[bool]
    delete: TMaybeRef[bool]
    code: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    editable: TMaybeRef[bool]
    editing: TMaybeRef[bool]
    default_editing: TMaybeRef[bool]
    edit_text: TMaybeRef[str]
    copyable: TMaybeRef[bool]
    copy_text: TMaybeRef[str]
    copy_delay: TMaybeRef[int]
    ellipsis: TMaybeRef[Union[bool, Any]]
    edit_tooltip_props: TMaybeRef[Dict]
    copy_tooltip_props: TMaybeRef[Dict]


class TDivider(TypedDict, total=False):
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    orientation: TMaybeRef[Union[str, Literal["left", "center", "right"]]]
    type: TMaybeRef[Union[str, Literal["solid", "dashed", "dotted", "double"]]]
    size: TMaybeRef[int]
    margin: TMaybeRef[Union[int, str]]


class TLayout(TypedDict, total=False):
    has_sider: TMaybeRef[bool]


class TSpace(TypedDict, total=False):
    align: TMaybeRef[Union[str, Literal["start", "end", "center", "baseline"]]]
    direction: TMaybeRef[Union[str, Literal["vertical", "horizontal"]]]
    size: TMaybeRef[Union[int, Union[str, Literal["mini", "small", "medium", "large"]]]]
    wrap: TMaybeRef[bool]
    fill: TMaybeRef[bool]


class TAvatar(TypedDict, total=False):
    shape: TMaybeRef[Union[str, Literal["circle", "square"]]]
    image_url: TMaybeRef[str]
    size: TMaybeRef[int]
    auto_fix_font_size: TMaybeRef[bool]
    trigger_type: TMaybeRef[Union[str, Literal["mask", "button"]]]
    trigger_icon_style: Any
    object_fit: TMaybeRef[
        Union[str, Literal["fill", "contain", "cover", "none", "scale-down"]]
    ]


class TBadge(TypedDict, total=False):
    text: TMaybeRef[str]
    dot: TMaybeRef[bool]
    dot_style: TMaybeRef[Dict]
    max_count: TMaybeRef[int]
    offset: TMaybeRef[List[int]]
    color: TMaybeRef[Union[Any, str]]
    status: TMaybeRef[
        Union[str, Literal["normal", "processing", "success", "warning", "danger"]]
    ]
    count: TMaybeRef[int]


class TCalendar(TypedDict, total=False):
    model_value: TMaybeRef[datetime.date]
    default_value: TMaybeRef[datetime.date]
    mode: TMaybeRef[Union[str, Literal["month", "year"]]]
    default_mode: TMaybeRef[Union[str, Literal["month", "year"]]]
    modes: TMaybeRef[List[Union[str, Literal["month", "year"]]]]


class TCard(TypedDict, total=False):
    bordered: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    hoverable: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["medium", "small"]]]
    header_style: Any
    body_style: Any
    title: TMaybeRef[str]
    extra: TMaybeRef[str]


class TCarousel(TypedDict, total=False):
    current: TMaybeRef[int]
    default_current: TMaybeRef[int]
    auto_play: TMaybeRef[Union[bool, Any]]
    move_speed: TMaybeRef[int]
    animation_name: TMaybeRef[Union[str, Literal["slide", "fade", "card"]]]
    trigger: TMaybeRef[Union[str, Literal["click", "hover"]]]
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    show_arrow: TMaybeRef[Union[str, Literal["always", "hover", "never"]]]
    arrow_class: TMaybeRef[str]
    indicator_type: TMaybeRef[Union[str, Literal["line", "dot", "slider", "never"]]]
    indicator_position: TMaybeRef[
        Union[str, Literal["bottom", "top", "left", "right", "outer"]]
    ]
    indicator_class: TMaybeRef[str]
    transition_timing_function: TMaybeRef[str]


class TCollapse(TypedDict, total=False):
    active_key: TMaybeRef[List[Union[str, int]]]
    default_active_key: TMaybeRef[List[Union[str, int]]]
    accordion: TMaybeRef[bool]
    show_expand_icon: TMaybeRef[bool]
    expand_icon_position: TMaybeRef[Union[str, Literal["left", "right"]]]
    bordered: TMaybeRef[bool]
    destroy_on_hide: TMaybeRef[bool]


class TComment(TypedDict, total=False):
    author: TMaybeRef[str]
    avatar: TMaybeRef[str]
    content: TMaybeRef[str]
    datetime: TMaybeRef[str]
    align: Any


class TDescriptions(TypedDict, total=False):
    data: TMaybeRef[List]
    column: TMaybeRef[Union[int, Any]]
    title: TMaybeRef[str]
    layout: TMaybeRef[
        Union[
            str,
            Literal["horizontal", "vertical", "inline-horizontal", "inline-vertical"],
        ]
    ]
    align: Any
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    bordered: TMaybeRef[bool]
    label_style: Any
    value_style: Any
    table_layout: TMaybeRef[Union[str, Literal["auto", "fixed"]]]


class TEmpty(TypedDict, total=False):
    description: TMaybeRef[str]
    img_src: TMaybeRef[str]
    in_config_provider: TMaybeRef[bool]


class TImage(TypedDict, total=False):
    src: TMaybeRef[str]
    width: TMaybeRef[Union[str, int]]
    height: TMaybeRef[Union[str, int]]
    title: TMaybeRef[str]
    description: TMaybeRef[str]
    fit: TMaybeRef[
        Union[str, Literal["contain", "cover", "fill", "none", "scale-down"]]
    ]
    alt: TMaybeRef[str]
    hide_footer: TMaybeRef[Union[bool, Union[str, Literal["never"]]]]
    footer_position: TMaybeRef[Union[str, Literal["inner", "outer"]]]
    show_loader: TMaybeRef[bool]
    preview: TMaybeRef[bool]
    preview_visible: TMaybeRef[bool]
    default_preview_visible: TMaybeRef[bool]
    preview_props: Any
    footer_class: TMaybeRef[Union[str, List, Dict]]


class TList(TypedDict, total=False):
    data: TMaybeRef[List]
    size: TMaybeRef[Union[str, Literal["small", "medium", "large"]]]
    bordered: TMaybeRef[bool]
    split: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    hoverable: TMaybeRef[bool]
    pagination_props: Any
    grid_props: TMaybeRef[Dict]
    max_height: TMaybeRef[Union[str, int]]
    bottom_offset: TMaybeRef[int]
    virtual_list_props: Any
    scrollbar: TMaybeRef[Union[bool, Any]]


class TPopover(TypedDict, total=False):
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    title: TMaybeRef[str]
    content: TMaybeRef[str]
    trigger: TMaybeRef[Union[str, Literal["hover", "click", "focus", "contextMenu"]]]
    position: TMaybeRef[
        Union[
            str,
            Literal[
                "top",
                "tl",
                "tr",
                "bottom",
                "bl",
                "br",
                "left",
                "lt",
                "lb",
                "right",
                "rt",
                "rb",
            ],
        ]
    ]
    content_class: Any
    content_style: Any
    arrow_class: Any
    arrow_style: Any
    popup_container: TMaybeRef[Union[str, Any]]


class TStatistic(TypedDict, total=False):
    title: TMaybeRef[str]
    value: TMaybeRef[Union[int, datetime.date, datetime.datetime]]
    format: TMaybeRef[str]
    extra: TMaybeRef[str]
    start: TMaybeRef[bool]
    precision: TMaybeRef[int]
    separator: TMaybeRef[str]
    show_group_separator: TMaybeRef[bool]
    animation: TMaybeRef[bool]
    animation_duration: TMaybeRef[int]
    value_from_: TMaybeRef[int]
    placeholder: TMaybeRef[str]
    value_style: Any


class TTable(TypedDict, total=False):
    columns: TMaybeRef[List]
    data: TMaybeRef[List]
    bordered: TMaybeRef[Union[bool, Any]]
    hoverable: TMaybeRef[bool]
    stripe: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    table_layout_fixed: TMaybeRef[bool]
    loading: TMaybeRef[Union[bool, Dict[str, Any]]]
    row_selection: Any
    expandable: Any
    scroll: TMaybeRef[Dict[str, Union[int, str]]]
    pagination: TMaybeRef[Union[bool, Any]]
    page_position: TMaybeRef[
        Union[str, Literal["tl", "top", "tr", "bl", "bottom", "br"]]
    ]
    indent_size: TMaybeRef[int]
    row_key: TMaybeRef[str]
    show_header: TMaybeRef[bool]
    virtual_list_props: Any
    span_method: Any
    span_all: TMaybeRef[bool]
    load_more: TMaybeRef[str]
    filter_icon_align_left: TMaybeRef[bool]
    hide_expand_button_on_empty: TMaybeRef[bool]
    row_class: TMaybeRef[Union[str, List]]
    draggable: Any
    column_resizable: TMaybeRef[bool]
    summary: TMaybeRef[Union[bool, str]]
    summary_text: TMaybeRef[str]
    summary_span_method: Any
    selected_keys: TMaybeRef[List[Union[str, int]]]
    default_selected_keys: TMaybeRef[List[Union[str, int]]]
    expanded_keys: TMaybeRef[List[Union[str, int]]]
    default_expanded_keys: TMaybeRef[List[Union[str, int]]]
    default_expand_all_rows: TMaybeRef[bool]
    sticky_header: TMaybeRef[Union[bool, int]]
    scrollbar: TMaybeRef[Union[bool, Any]]
    show_empty_tree: TMaybeRef[bool]


class TTabs(TypedDict, total=False):
    default_active_key: TMaybeRef[Union[str, int]]
    position: TMaybeRef[Union[str, Literal["left", "right", "top", "bottom"]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    type: TMaybeRef[
        Union[str, Literal["line", "card", "card-gutter", "text", "rounded", "capsule"]]
    ]
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    editable: TMaybeRef[bool]
    show_add_button: TMaybeRef[bool]
    destroy_on_hide: TMaybeRef[bool]
    lazy_load: TMaybeRef[bool]
    justify: TMaybeRef[bool]
    animation: TMaybeRef[bool]
    header_padding: TMaybeRef[bool]
    auto_switch: TMaybeRef[bool]
    hide_content: TMaybeRef[bool]
    trigger: TMaybeRef[Union[str, Literal["hover", "click"]]]
    scroll_position: TMaybeRef[
        Union[Union[str, Literal["start", "end", "center", "auto"]], int]
    ]


class TTag(TypedDict, total=False):
    color: TMaybeRef[
        Union[
            str,
            Literal[
                "red",
                "orangered",
                "orange",
                "gold",
                "lime",
                "green",
                "cyan",
                "blue",
                "arcoblue",
                "purple",
                "pinkpurple",
                "magenta",
                "gray",
            ],
        ]
    ]
    size: TMaybeRef[Union[str, Literal["small", "medium", "large"]]]
    bordered: TMaybeRef[bool]
    visible: TMaybeRef[bool]
    default_visible: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    closable: TMaybeRef[bool]
    checkable: TMaybeRef[bool]
    checked: TMaybeRef[bool]
    default_checked: TMaybeRef[bool]
    nowrap: TMaybeRef[bool]


class TTimeline(TypedDict, total=False):
    reverse: TMaybeRef[bool]
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    mode: TMaybeRef[Union[str, Literal["left", "right", "top", "bottom", "alternate"]]]
    pending: TMaybeRef[Union[bool, str]]
    label_position: TMaybeRef[Union[str, Literal["relative", "same"]]]


class TTooltip(TypedDict, total=False):
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    content: TMaybeRef[str]
    position: TMaybeRef[
        Union[
            str,
            Literal[
                "top",
                "tl",
                "tr",
                "bottom",
                "bl",
                "br",
                "left",
                "lt",
                "lb",
                "right",
                "rt",
                "rb",
            ],
        ]
    ]
    mini: TMaybeRef[bool]
    background_color: TMaybeRef[str]
    content_class: Any
    content_style: Any
    arrow_class: Any
    arrow_style: Any
    popup_container: TMaybeRef[Union[str, Any]]


class TTree(TypedDict, total=False):
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    block_node: TMaybeRef[bool]
    default_expand_all: TMaybeRef[bool]
    multiple: TMaybeRef[bool]
    checkable: TMaybeRef[Union[bool, str]]
    selectable: TMaybeRef[Union[bool, str]]
    check_strictly: TMaybeRef[bool]
    checked_strategy: TMaybeRef[Union[str, Literal["all", "parent", "child"]]]
    default_selected_keys: TMaybeRef[List[Union[str, int]]]
    selected_keys: TMaybeRef[List[Union[str, int]]]
    default_checked_keys: TMaybeRef[List[Union[str, int]]]
    checked_keys: TMaybeRef[List[Union[str, int]]]
    default_expanded_keys: TMaybeRef[List[Union[str, int]]]
    expanded_keys: TMaybeRef[List[Union[str, int]]]
    data: TMaybeRef[List]
    field_names: Any
    show_line: TMaybeRef[bool]
    load_more: TMaybeRef[str]
    draggable: TMaybeRef[bool]
    allow_drop: TMaybeRef[str]
    virtual_list_props: Any
    default_expand_selected: TMaybeRef[bool]
    default_expand_checked: TMaybeRef[bool]
    auto_expand_parent: TMaybeRef[bool]
    half_checked_keys: TMaybeRef[List[Union[str, int]]]
    only_check_leaf: TMaybeRef[bool]
    animation: TMaybeRef[bool]
    action_on_node_click: TMaybeRef[Union[str, Literal["expand"]]]


class TAutocomplete(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    disabled: TMaybeRef[bool]
    data: TMaybeRef[List[Union[str, int, Any]]]
    popup_container: TMaybeRef[Union[str, Any, None, Any]]
    strict: TMaybeRef[bool]
    filter_option: Any
    trigger_props: Any
    allow_clear: TMaybeRef[bool]
    virtual_list_props: Any


class TCascader(TypedDict, total=False):
    path_mode: TMaybeRef[bool]
    multiple: TMaybeRef[bool]
    model_value: TMaybeRef[Union[str, int, Dict[str, Any]]]
    default_value: TMaybeRef[Union[str, int, Dict[str, Any]]]
    disabled: TMaybeRef[bool]
    error: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    allow_search: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    input_value: TMaybeRef[str]
    default_input_value: TMaybeRef[str]
    popup_visible: TMaybeRef[bool]
    expand_trigger: TMaybeRef[Union[str, Literal["click", "hover"]]]
    default_popup_visible: TMaybeRef[bool]
    placeholder: TMaybeRef[str]
    filter_option: TMaybeRef[str]
    popup_container: TMaybeRef[Union[str, Any]]
    max_tag_count: TMaybeRef[int]
    format_label: TMaybeRef[str]
    trigger_props: Any
    check_strictly: TMaybeRef[bool]
    load_more: TMaybeRef[str]
    loading: TMaybeRef[bool]
    search_option_only_label: TMaybeRef[bool]
    search_delay: TMaybeRef[int]
    field_names: Any
    value_key: TMaybeRef[str]
    fallback: TMaybeRef[Union[bool, str]]
    expand_child: TMaybeRef[bool]
    virtual_list_props: Any
    tag_nowrap: TMaybeRef[bool]


class TCheckbox(TypedDict, total=False):
    model_value: TMaybeRef[Union[bool, List[Union[str, int, bool]]]]
    default_checked: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    indeterminate: TMaybeRef[bool]


class TColorpicker(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    format: TMaybeRef[Union[str, Literal["hex", "rgb"]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    show_text: TMaybeRef[bool]
    show_history: TMaybeRef[bool]
    show_preset: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    disabled_alpha: TMaybeRef[bool]
    hide_trigger: TMaybeRef[bool]
    trigger_props: Any
    history_colors: TMaybeRef[List[str]]
    preset_colors: TMaybeRef[List[str]]


class TDatepicker(TypedDict, total=False):
    locale: TMaybeRef[Dict[str, Any]]
    hide_trigger: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    error: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    shortcuts: TMaybeRef[List]
    shortcuts_position: TMaybeRef[Union[str, Literal["left", "bottom", "right"]]]
    position: TMaybeRef[Union[str, Literal["top", "tl", "tr", "bottom", "bl", "br"]]]
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    trigger_props: Any
    unmount_on_close: TMaybeRef[bool]
    placeholder: TMaybeRef[str]
    disabled: TMaybeRef[bool]
    disabled_date: TMaybeRef[str]
    disabled_time: TMaybeRef[str]
    picker_value: TMaybeRef[Union[datetime.date, datetime.datetime, str, int]]
    default_picker_value: TMaybeRef[Union[datetime.date, datetime.datetime, str, int]]
    popup_container: TMaybeRef[Union[str, Any]]
    value_format: TMaybeRef[Union[Union[str, Literal["timestamp", "Date"]], str]]
    preview_shortcut: TMaybeRef[bool]
    show_confirm_btn: TMaybeRef[bool]
    disabled_input: TMaybeRef[bool]
    abbreviation: TMaybeRef[bool]


class TForm(TypedDict, total=False):
    model: TMaybeRef[Dict]
    layout: TMaybeRef[Union[str, Literal["horizontal", "vertical", "inline"]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    label_col_props: TMaybeRef[Dict]
    wrapper_col_props: TMaybeRef[Dict]
    label_align: TMaybeRef[Union[str, Literal["left", "right"]]]
    disabled: TMaybeRef[bool]
    rules: TMaybeRef[Dict[str, Union[Any, List]]]
    auto_label_width: TMaybeRef[bool]
    id: TMaybeRef[str]
    scroll_to_first_error: TMaybeRef[bool]


class TInput(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    allow_clear: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    error: TMaybeRef[bool]
    placeholder: TMaybeRef[str]
    max_length: TMaybeRef[Union[int, Dict[str, Union[int, bool]]]]
    show_word_limit: TMaybeRef[bool]
    word_length: TMaybeRef[str]
    word_slice: TMaybeRef[str]
    input_attrs: TMaybeRef[Dict]
    prepend: TMaybeRef[str]
    append: TMaybeRef[str]


class TInputnumber(TypedDict, total=False):
    model_value: TMaybeRef[int]
    default_value: TMaybeRef[int]
    mode: TMaybeRef[Union[str, Literal["embed", "button"]]]
    precision: TMaybeRef[int]
    step: TMaybeRef[int]
    disabled: TMaybeRef[bool]
    error: TMaybeRef[bool]
    max: TMaybeRef[int]
    min: TMaybeRef[int]
    formatter: TMaybeRef[str]
    parser: TMaybeRef[str]
    placeholder: TMaybeRef[str]
    hide_button: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    allow_clear: TMaybeRef[bool]
    model_event: TMaybeRef[Union[str, Literal["change", "input"]]]
    read_only: TMaybeRef[bool]
    input_attrs: TMaybeRef[Dict]


class TVerificationcode(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    length: TMaybeRef[int]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    disabled: TMaybeRef[bool]
    masked: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    error: TMaybeRef[bool]
    separator: TMaybeRef[str]
    formatter: TMaybeRef[str]


class TInputtag(TypedDict, total=False):
    model_value: TMaybeRef[List[Union[str, int, Any]]]
    default_value: TMaybeRef[List[Union[str, int, Any]]]
    input_value: TMaybeRef[str]
    default_input_value: TMaybeRef[str]
    placeholder: TMaybeRef[str]
    disabled: TMaybeRef[bool]
    error: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    max_tag_count: TMaybeRef[int]
    retain_input_value: TMaybeRef[Union[bool, Dict[str, bool]]]
    format_tag: TMaybeRef[str]
    unique_value: TMaybeRef[bool]
    field_names: Any
    tag_nowrap: TMaybeRef[bool]


class TMention(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    data: TMaybeRef[List[Union[str, int, Any]]]
    prefix: TMaybeRef[Union[str, List[str]]]
    split: TMaybeRef[str]
    type: TMaybeRef[Union[str, Literal["input", "textarea"]]]
    disabled: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]


class TRadio(TypedDict, total=False):
    model_value: TMaybeRef[Union[str, int, bool]]
    default_checked: TMaybeRef[bool]
    type: TMaybeRef[Union[str, Literal["radio", "button"]]]
    disabled: TMaybeRef[bool]


class TRate(TypedDict, total=False):
    count: TMaybeRef[int]
    model_value: TMaybeRef[int]
    default_value: TMaybeRef[int]
    allow_half: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    grading: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    color: TMaybeRef[Union[str, Dict[str, str]]]


class TSelect(TypedDict, total=False):
    multiple: TMaybeRef[bool]
    model_value: TMaybeRef[
        Union[
            str, int, bool, Dict[str, Any], List[Union[str, int, bool, Dict[str, Any]]]
        ]
    ]
    default_value: TMaybeRef[
        Union[
            str, int, bool, Dict[str, Any], List[Union[str, int, bool, Dict[str, Any]]]
        ]
    ]
    input_value: TMaybeRef[str]
    default_input_value: TMaybeRef[str]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    placeholder: TMaybeRef[str]
    loading: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    error: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    allow_search: TMaybeRef[Union[bool, Dict[str, bool]]]
    allow_create: TMaybeRef[bool]
    max_tag_count: TMaybeRef[int]
    popup_container: TMaybeRef[Union[str, Any]]
    bordered: TMaybeRef[bool]
    default_active_first_option: TMaybeRef[bool]
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    unmount_on_close: TMaybeRef[bool]
    filter_option: TMaybeRef[Union[bool, str]]
    virtual_list_props: Any
    trigger_props: Any
    format_label: TMaybeRef[str]
    fallback_option: TMaybeRef[Union[bool, str]]
    show_extra_options: TMaybeRef[bool]
    value_key: TMaybeRef[str]
    search_delay: TMaybeRef[int]
    limit: TMaybeRef[int]
    field_names: Any
    scrollbar: TMaybeRef[Union[bool, Any]]
    show_header_on_empty: TMaybeRef[bool]
    show_footer_on_empty: TMaybeRef[bool]
    tag_nowrap: TMaybeRef[bool]


class TSlider(TypedDict, total=False):
    model_value: TMaybeRef[Union[int, Tuple[int, int]]]
    default_value: TMaybeRef[Union[int, Tuple[int, int]]]
    step: TMaybeRef[int]
    min: TMaybeRef[int]
    marks: TMaybeRef[Dict[int, str]]
    max: TMaybeRef[int]
    direction: Any
    disabled: TMaybeRef[bool]
    show_ticks: TMaybeRef[bool]
    show_input: TMaybeRef[bool]
    range: TMaybeRef[bool]
    show_tooltip: TMaybeRef[bool]


class TSwitch(TypedDict, total=False):
    model_value: TMaybeRef[Union[str, int, bool]]
    default_checked: TMaybeRef[bool]
    disabled: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    type: TMaybeRef[Union[str, Literal["circle", "round", "line"]]]
    size: TMaybeRef[Union[str, Literal["small", "medium"]]]
    checked_value: TMaybeRef[Union[str, int, bool]]
    unchecked_value: TMaybeRef[Union[str, int, bool]]
    checked_color: TMaybeRef[str]
    unchecked_color: TMaybeRef[str]
    before_change: TMaybeRef[str]
    checked_text: TMaybeRef[str]
    unchecked_text: TMaybeRef[str]


class TTextarea(TypedDict, total=False):
    model_value: TMaybeRef[str]
    default_value: TMaybeRef[str]
    placeholder: TMaybeRef[str]
    disabled: TMaybeRef[bool]
    error: TMaybeRef[bool]
    max_length: TMaybeRef[Union[int, Dict[str, Union[int, bool]]]]
    show_word_limit: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    auto_size: TMaybeRef[Union[bool, Dict[str, Optional[int]]]]
    word_length: TMaybeRef[str]
    word_slice: TMaybeRef[str]
    textarea_attrs: TMaybeRef[Dict[str, Any]]


class TTimepicker(TypedDict, total=False):
    type: TMaybeRef[Union[str, Literal["time", "time-range"]]]
    model_value: TMaybeRef[
        Union[str, int, datetime.date, List[Union[str, int, datetime.date]]]
    ]
    default_value: TMaybeRef[
        Union[str, int, datetime.date, List[Union[str, int, datetime.date]]]
    ]
    disabled: TMaybeRef[bool]
    allow_clear: TMaybeRef[bool]
    readonly: TMaybeRef[bool]
    error: TMaybeRef[bool]
    format: TMaybeRef[str]
    placeholder: TMaybeRef[Union[str, List[str]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    popup_container: TMaybeRef[Union[str, Any]]
    use12_hours: TMaybeRef[bool]
    step: TMaybeRef[Dict[str, int]]
    disabled_hours: TMaybeRef[str]
    disabled_minutes: TMaybeRef[str]
    disabled_seconds: Any
    hide_disabled_options: TMaybeRef[bool]
    disable_confirm: TMaybeRef[bool]
    position: TMaybeRef[Union[str, Literal["top", "tl", "tr", "bottom", "bl", "br"]]]
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    trigger_props: Any
    unmount_on_close: TMaybeRef[bool]


class TTransfer(TypedDict, total=False):
    data: TMaybeRef[List]
    model_value: TMaybeRef[List[str]]
    default_value: TMaybeRef[List[str]]
    selected: TMaybeRef[List[str]]
    default_selected: TMaybeRef[List[str]]
    disabled: TMaybeRef[bool]
    simple: TMaybeRef[bool]
    one_way: TMaybeRef[bool]
    show_search: TMaybeRef[bool]
    show_select_all: TMaybeRef[bool]
    title: TMaybeRef[List[str]]
    source_input_search_props: TMaybeRef[Dict]
    target_input_search_props: TMaybeRef[Dict]


class TTreeselect(TypedDict, total=False):
    disabled: TMaybeRef[bool]
    loading: TMaybeRef[bool]
    error: TMaybeRef[bool]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    border: TMaybeRef[bool]
    allow_search: TMaybeRef[Union[bool, Dict[str, bool]]]
    allow_clear: TMaybeRef[bool]
    placeholder: TMaybeRef[str]
    max_tag_count: TMaybeRef[int]
    multiple: TMaybeRef[bool]
    default_value: TMaybeRef[Union[str, int, List[Union[str, int]], Any]]
    model_value: TMaybeRef[Union[str, int, List[Union[str, int]], Any]]
    field_names: Any
    data: TMaybeRef[List]
    label_in_value: TMaybeRef[bool]
    tree_checkable: TMaybeRef[bool]
    tree_check_strictly: TMaybeRef[bool]
    tree_checked_strategy: TMaybeRef[Union[str, Literal["all", "parent", "child"]]]
    tree_props: Any
    trigger_props: Any
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    dropdown_style: Any
    dropdown_class_name: TMaybeRef[Union[str, List[str]]]
    filter_tree_node: TMaybeRef[str]
    load_more: TMaybeRef[str]
    disable_filter: TMaybeRef[bool]
    popup_container: TMaybeRef[Union[str, Any]]
    fallback_option: TMaybeRef[Union[bool, str]]
    selectable: TMaybeRef[Union[bool, Union[str, Literal["leaf"]]]]
    scrollbar: TMaybeRef[Union[bool, Any]]
    show_header_on_empty: TMaybeRef[bool]
    show_footer_on_empty: TMaybeRef[bool]
    input_value: TMaybeRef[str]
    default_input_value: TMaybeRef[str]


class TUpload(TypedDict, total=False):
    file_list: TMaybeRef[List]
    default_file_list: TMaybeRef[List]
    accept: TMaybeRef[str]
    action: TMaybeRef[str]
    disabled: TMaybeRef[bool]
    multiple: TMaybeRef[bool]
    directory: TMaybeRef[bool]
    draggable: TMaybeRef[bool]
    tip: TMaybeRef[str]
    headers: TMaybeRef[Dict[str, str]]
    data: TMaybeRef[Union[Dict[str, Union[str, Any]], str]]
    name: TMaybeRef[Union[str, str]]
    with_credentials: TMaybeRef[bool]
    custom_request: TMaybeRef[str]
    limit: TMaybeRef[int]
    auto_upload: TMaybeRef[bool]
    show_file_list: TMaybeRef[bool]
    show_remove_button: TMaybeRef[bool]
    show_retry_button: TMaybeRef[bool]
    show_cancel_button: TMaybeRef[bool]
    show_upload_button: TMaybeRef[Union[bool, Dict[str, bool]]]
    show_preview_button: TMaybeRef[bool]
    download: TMaybeRef[bool]
    show_link: TMaybeRef[bool]
    image_loading: TMaybeRef[Union[str, Literal["eager", "lazy"]]]
    list_type: TMaybeRef[Union[str, Literal["text", "picture", "picture-card"]]]
    response_url_key: TMaybeRef[Union[str, str]]
    custom_icon: Any
    image_preview: TMaybeRef[bool]
    on_before_upload: TMaybeRef[str]
    on_before_remove: TMaybeRef[str]
    on_button_click: TMaybeRef[str]


class TAlert(TypedDict, total=False):
    type: TMaybeRef[
        Union[str, Literal["info", "success", "warning", "error", "normal"]]
    ]
    show_icon: TMaybeRef[bool]
    closable: TMaybeRef[bool]
    title: TMaybeRef[str]
    banner: TMaybeRef[bool]
    center: TMaybeRef[bool]


class TDrawer(TypedDict, total=False):
    visible: TMaybeRef[bool]
    default_visible: TMaybeRef[bool]
    placement: TMaybeRef[Union[str, Literal["top", "right", "bottom", "left"]]]
    title: TMaybeRef[str]
    mask: TMaybeRef[bool]
    mask_closable: TMaybeRef[bool]
    closable: TMaybeRef[bool]
    ok_text: TMaybeRef[str]
    cancel_text: TMaybeRef[str]
    ok_loading: TMaybeRef[bool]
    ok_button_props: Any
    cancel_button_props: Any
    unmount_on_close: TMaybeRef[bool]
    width: TMaybeRef[Union[int, str]]
    height: TMaybeRef[Union[int, str]]
    popup_container: TMaybeRef[Union[str, Any]]
    drawer_style: Any
    body_class: TMaybeRef[Union[str, List]]
    body_style: Any
    on_before_ok: TMaybeRef[str]
    on_before_cancel: TMaybeRef[str]
    esc_to_close: TMaybeRef[bool]
    render_to_body: TMaybeRef[bool]
    header: TMaybeRef[bool]
    footer: TMaybeRef[bool]
    hide_cancel: TMaybeRef[bool]


class TModal(TypedDict, total=False):
    visible: TMaybeRef[bool]
    default_visible: TMaybeRef[bool]
    width: TMaybeRef[Union[int, str]]
    top: TMaybeRef[Union[int, str]]
    mask: TMaybeRef[bool]
    title: TMaybeRef[str]
    title_align: TMaybeRef[Union[str, Literal["start", "center"]]]
    align_center: TMaybeRef[bool]
    unmount_on_close: TMaybeRef[bool]
    mask_closable: TMaybeRef[bool]
    hide_cancel: TMaybeRef[bool]
    simple: TMaybeRef[bool]
    closable: TMaybeRef[bool]
    ok_text: TMaybeRef[str]
    cancel_text: TMaybeRef[str]
    ok_loading: TMaybeRef[bool]
    ok_button_props: Any
    cancel_button_props: Any
    footer: TMaybeRef[bool]
    render_to_body: TMaybeRef[bool]
    popup_container: TMaybeRef[Union[str, Any]]
    mask_style: Any
    modal_class: TMaybeRef[Union[str, List]]
    modal_style: Any
    on_before_ok: TMaybeRef[str]
    on_before_cancel: TMaybeRef[str]
    esc_to_close: TMaybeRef[bool]
    draggable: TMaybeRef[bool]
    fullscreen: TMaybeRef[bool]
    mask_animation_name: TMaybeRef[str]
    modal_animation_name: TMaybeRef[str]
    body_class: TMaybeRef[Union[str, List]]
    body_style: Any
    hide_title: TMaybeRef[bool]


class TNotification(TypedDict, total=False):
    info: TMaybeRef[str]
    success: TMaybeRef[str]
    warning: TMaybeRef[str]
    error: TMaybeRef[str]
    remove: TMaybeRef[str]
    clear: TMaybeRef[str]


class TPopconfirm(TypedDict, total=False):
    content: TMaybeRef[str]
    position: TMaybeRef[
        Union[
            str,
            Literal[
                "top",
                "tl",
                "tr",
                "bottom",
                "bl",
                "br",
                "left",
                "lt",
                "lb",
                "right",
                "rt",
                "rb",
            ],
        ]
    ]
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    type: TMaybeRef[Union[str, Literal["info", "success", "warning", "error"]]]
    ok_text: TMaybeRef[str]
    cancel_text: TMaybeRef[str]
    ok_loading: TMaybeRef[bool]
    ok_button_props: Any
    cancel_button_props: Any
    content_class: Any
    content_style: Any
    arrow_class: Any
    arrow_style: Any
    popup_container: TMaybeRef[Union[str, Any]]
    on_before_ok: TMaybeRef[str]
    on_before_cancel: TMaybeRef[str]


class TProgress(TypedDict, total=False):
    type: TMaybeRef[Union[str, Literal["line", "circle"]]]
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    percent: TMaybeRef[int]
    steps: TMaybeRef[int]
    animation: TMaybeRef[bool]
    stroke_width: TMaybeRef[int]
    width: TMaybeRef[Union[int, str]]
    color: TMaybeRef[Union[str, Dict]]
    track_color: TMaybeRef[str]
    show_text: TMaybeRef[bool]
    status: TMaybeRef[Union[str, Literal["normal", "success", "warning", "danger"]]]


class TResult(TypedDict, total=False):
    status: TMaybeRef[
        Union[
            Union[
                str, Literal["info", "success", "warning", "error", "403", "404", "500"]
            ],
            None,
        ]
    ]
    title: TMaybeRef[str]
    subtitle: TMaybeRef[str]


class TSpin(TypedDict, total=False):
    size: TMaybeRef[int]
    loading: TMaybeRef[bool]
    dot: TMaybeRef[bool]
    tip: TMaybeRef[str]
    hide_icon: TMaybeRef[bool]


class TSkeleton(TypedDict, total=False):
    loading: TMaybeRef[bool]
    animation: TMaybeRef[bool]


class TBreadcrumb(TypedDict, total=False):
    max_count: TMaybeRef[int]
    routes: TMaybeRef[List]
    separator: TMaybeRef[Union[str, int]]
    custom_url: TMaybeRef[str]


class TDropdown(TypedDict, total=False):
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    trigger: TMaybeRef[Union[str, Literal["hover", "click", "focus", "contextMenu"]]]
    position: TMaybeRef[Union[str, Literal["top", "tl", "tr", "bottom", "bl", "br"]]]
    popup_container: TMaybeRef[Union[str, Any]]
    popup_max_height: TMaybeRef[Union[bool, int]]
    hide_on_select: TMaybeRef[bool]


class TMenu(TypedDict, total=False):
    theme: TMaybeRef[Union[str, Literal["light", "dark"]]]
    mode: TMaybeRef[Union[str, Literal["vertical", "horizontal", "pop", "popButton"]]]
    level_indent: TMaybeRef[int]
    auto_open: TMaybeRef[bool]
    collapsed: TMaybeRef[bool]
    default_collapsed: TMaybeRef[bool]
    collapsed_width: TMaybeRef[int]
    accordion: TMaybeRef[bool]
    auto_scroll_into_view: TMaybeRef[bool]
    show_collapse_button: TMaybeRef[bool]
    selected_keys: TMaybeRef[List[str]]
    default_selected_keys: TMaybeRef[List[str]]
    open_keys: TMaybeRef[List[str]]
    default_open_keys: TMaybeRef[List[str]]
    scroll_config: TMaybeRef[Dict[str, Any]]
    trigger_props: Any
    tooltip_props: TMaybeRef[Dict]
    auto_open_selected: TMaybeRef[bool]
    breakpoint: TMaybeRef[Union[str, Literal["xxl", "xl", "lg", "md", "sm", "xs"]]]
    popup_max_height: TMaybeRef[Union[bool, int]]


class TPageheader(TypedDict, total=False):
    title: TMaybeRef[str]
    subtitle: TMaybeRef[str]
    show_back: TMaybeRef[bool]


class TPagination(TypedDict, total=False):
    total: TMaybeRef[int]
    current: TMaybeRef[int]
    default_current: TMaybeRef[int]
    page_size: TMaybeRef[int]
    default_page_size: TMaybeRef[int]
    disabled: TMaybeRef[bool]
    hide_on_single_page: TMaybeRef[bool]
    simple: TMaybeRef[bool]
    show_total: TMaybeRef[bool]
    show_more: TMaybeRef[bool]
    show_jumper: TMaybeRef[bool]
    show_page_size: TMaybeRef[bool]
    page_size_options: TMaybeRef[List[int]]
    page_size_props: Any
    size: TMaybeRef[Union[str, Literal["mini", "small", "medium", "large"]]]
    page_item_style: Any
    active_page_item_style: Any
    base_size: TMaybeRef[int]
    buffer_size: TMaybeRef[int]
    auto_adjust: TMaybeRef[bool]


class TSteps(TypedDict, total=False):
    type: TMaybeRef[Union[str, Literal["default", "arrow", "dot", "navigation"]]]
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    label_placement: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    current: TMaybeRef[int]
    default_current: TMaybeRef[int]
    status: TMaybeRef[Union[str, Literal["wait", "process", "finish", "error"]]]
    line_less: TMaybeRef[bool]
    small: TMaybeRef[bool]
    changeable: TMaybeRef[bool]


class TAffix(TypedDict, total=False):
    offset_top: TMaybeRef[int]
    offset_bottom: TMaybeRef[int]
    target: TMaybeRef[Union[str, Any, Any]]
    target_container: TMaybeRef[Union[str, Any, Any]]


class TAnchor(TypedDict, total=False):
    boundary: TMaybeRef[
        Union[Union[str, Literal["start", "end", "center", "nearest"]], int]
    ]
    line_less: TMaybeRef[bool]
    scroll_container: TMaybeRef[Union[str, Any, Any]]
    change_hash: TMaybeRef[bool]
    smooth: TMaybeRef[bool]


class TBacktop(TypedDict, total=False):
    visible_height: TMaybeRef[int]
    target_container: TMaybeRef[Union[str, Any]]
    easing: TMaybeRef[str]
    duration: TMaybeRef[int]


class TConfigprovider(TypedDict, total=False):
    prefix_cls: TMaybeRef[str]
    locale: TMaybeRef[Dict]
    size: Any
    global_: TMaybeRef[bool]
    update_at_scroll: TMaybeRef[bool]
    scroll_to_close: TMaybeRef[bool]
    exchange_time: TMaybeRef[bool]


class TResizebox(TypedDict, total=False):
    width: TMaybeRef[int]
    height: TMaybeRef[int]
    component: TMaybeRef[str]
    directions: TMaybeRef[List[Union[str, Literal["left", "right", "top", "bottom"]]]]


class TTrigger(TypedDict, total=False):
    popup_visible: TMaybeRef[bool]
    default_popup_visible: TMaybeRef[bool]
    trigger: TMaybeRef[Union[str, Literal["hover", "click", "focus", "contextMenu"]]]
    position: TMaybeRef[
        Union[
            str,
            Literal[
                "top",
                "tl",
                "tr",
                "bottom",
                "bl",
                "br",
                "left",
                "lt",
                "lb",
                "right",
                "rt",
                "rb",
            ],
        ]
    ]
    disabled: TMaybeRef[bool]
    popup_offset: TMaybeRef[int]
    popup_translate: Any
    show_arrow: TMaybeRef[bool]
    align_point: TMaybeRef[bool]
    popup_hover_stay: TMaybeRef[bool]
    blur_to_close: TMaybeRef[bool]
    click_to_close: TMaybeRef[bool]
    click_outside_to_close: TMaybeRef[bool]
    unmount_on_close: TMaybeRef[bool]
    content_class: TMaybeRef[Union[str, List, Dict]]
    content_style: Any
    arrow_class: TMaybeRef[Union[str, List, Dict]]
    arrow_style: Any
    popup_style: Any
    animation_name: TMaybeRef[str]
    duration: TMaybeRef[Union[int, Dict[str, int]]]
    mouse_enter_delay: TMaybeRef[int]
    mouse_leave_delay: TMaybeRef[int]
    focus_delay: TMaybeRef[int]
    auto_fit_popup_width: TMaybeRef[bool]
    auto_fit_popup_min_width: TMaybeRef[bool]
    auto_fix_position: TMaybeRef[bool]
    popup_container: TMaybeRef[Union[str, Any]]
    update_at_scroll: TMaybeRef[bool]
    auto_fit_position: TMaybeRef[bool]
    render_to_body: TMaybeRef[bool]
    prevent_focus: TMaybeRef[bool]
    scroll_to_close: TMaybeRef[bool]
    scroll_to_close_distance: TMaybeRef[int]


class TSplit(TypedDict, total=False):
    component: TMaybeRef[str]
    direction: TMaybeRef[Union[str, Literal["horizontal", "vertical"]]]
    size: TMaybeRef[Union[int, str]]
    default_size: TMaybeRef[Union[int, str]]
    min: TMaybeRef[Union[int, str]]
    max: TMaybeRef[Union[int, str]]
    disabled: TMaybeRef[bool]


class TOverflowlist(TypedDict, total=False):
    min: TMaybeRef[int]
    margin: TMaybeRef[int]
    from_: TMaybeRef[Union[str, Literal["start", "end"]]]


class TWatermark(TypedDict, total=False):
    content: TMaybeRef[Union[str, List[str]]]
    image: TMaybeRef[str]
    width: TMaybeRef[int]
    height: TMaybeRef[int]
    gap: TMaybeRef[Tuple[int, int]]
    offset: TMaybeRef[Tuple[int, int]]
    rotate: TMaybeRef[int]
    font: Any
    z_index: TMaybeRef[int]
    alpha: TMaybeRef[int]
    anti_tamper: TMaybeRef[bool]
    grayscale: TMaybeRef[bool]
    repeat: TMaybeRef[bool]
    staggered: TMaybeRef[bool]



from __future__ import annotations
from typing import List, Optional
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui.components.element import Element
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Link(Element):
    def __init__(
        self,
        text: Optional[TMaybeRef[str]] = None,
        **kwargs: Unpack[component_types.TLink],
    ):
        super().__init__("a-link")

        if text is not None:
            self.props({"text": text})

        self.props(handle_props(kwargs))  # type: ignore

    def on_click(
        self,
        handler: EventMixin,
        *,
        extends: Optional[List] = None,
        key: Optional[str] = None,
    ):
        self.on(
            "click",
            handler,
            extends=extends,
        )
        return self



from __future__ import annotations
from collections.abc import Sequence
import typing
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui import ui
from instaui.components.element import Element
from instaui.vars.mixin_types.observable import ObservableMixin
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props, try_setup_vmodel


_TSelectValue = typing.Union[str, int, typing.Sequence[str], typing.Sequence[int]]

_OPTIONS_TRANSLATE_JS: typing.Final = r"""(options)=>{
    if (Array.isArray(options)){
        const obj = {};
        options.forEach(item => {
            obj[item] = item;
        });
        return obj;                             
    }
    return options;
}
"""


class Select(Element):
    def __init__(
        self,
        options: TMaybeRef[typing.Union[typing.Sequence, typing.Dict]],
        value: typing.Optional[TMaybeRef[_TSelectValue]] = None,
        **kwargs: Unpack[component_types.TSelect],
    ):
        super().__init__("a-select")

        if isinstance(options, ObservableMixin):
            options = ui.js_computed(inputs=[options], code=_OPTIONS_TRANSLATE_JS)
        else:
            options = (
                {str(i): str(i) for i in options}
                if isinstance(options, Sequence)
                else options
            )

        with self.add_slot("default"):
            with ui.vfor(options) as item:
                item = ui.iter_info(item)
                Select.Option(item.dict_value).props(
                    {"value": item.dict_key, "label": item.dict_value}
                )

        try_setup_vmodel(self, value)
        self.props(handle_props(kwargs))  # type: ignore

    class Option(Element):
        def __init__(
            self,
            text: typing.Optional[TMaybeRef[str]] = None,
        ):
            super().__init__("a-option")

            if text is not None:
                with self:
                    ui.content(text)

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self

    def on_input_value_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "input-value-change",
            handler,
            extends=extends,
        )
        return self

    def on_popup_visible_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "popup-visible-change",
            handler,
            extends=extends,
        )
        return self

    def on_clear(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "clear",
            handler,
            extends=extends,
        )
        return self

    def on_remove(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "remove",
            handler,
            extends=extends,
        )
        return self

    def on_search(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "search",
            handler,
            extends=extends,
        )
        return self

    def on_dropdown_scroll(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "dropdown-scroll",
            handler,
            extends=extends,
        )
        return self

    def on_dropdown_reach_bottom(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "dropdown-reach-bottom",
            handler,
            extends=extends,
        )
        return self

    def on_exceed_limit(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "exceed-limit",
            handler,
            extends=extends,
        )
        return self



from __future__ import annotations
from pathlib import Path
from typing import Dict, Iterable, List, Literal, Optional
from instaui import ui
from instaui.runtime._app import get_app_slot
from instaui import consts

_STATIC_DIR = Path(__file__).parent / "static"
_THEME_DIR = _STATIC_DIR / "themes"
_LANG_DIR = _STATIC_DIR / "langs"
_SHIKI_CORE_FILE = _STATIC_DIR / "shiki-core.js"
_SHIKI_TRANSFORMERS_FILE = _STATIC_DIR / "shiki-transformers.js"
_STYLE = _STATIC_DIR / "shiki-style.css"


_LANGS_IMPORT_NAME = "@shiki/langs/"
_THEMES_IMPORT_NAME = "@shiki/themes/"

_IMPORT_MAPS = {
    "@shiki": _SHIKI_CORE_FILE,
    "@shiki/transformers": _SHIKI_TRANSFORMERS_FILE,
    _LANGS_IMPORT_NAME: _LANG_DIR,
    _THEMES_IMPORT_NAME: _THEME_DIR,
    "@instaui-tools/browser": consts.TOOLS_BROWSER_JS_PATH,
}

_ZERO_IMPORT_MAPS = {
    "@shiki": _SHIKI_CORE_FILE,
    "@shiki/transformers": _SHIKI_TRANSFORMERS_FILE,
    f"{_LANGS_IMPORT_NAME}python.mjs": _LANG_DIR / "python.mjs",
    f"{_THEMES_IMPORT_NAME}vitesse-light.mjs": _THEME_DIR / "vitesse-light.mjs",
    f"{_THEMES_IMPORT_NAME}vitesse-dark.mjs": _THEME_DIR / "vitesse-dark.mjs",
    "@instaui-tools/browser": consts.TOOLS_BROWSER_JS_PATH,
}


class Code(
    ui.element,
    esm="./shiki_code.js",
    externals=_IMPORT_MAPS,
    css=[_STYLE],
):
    # _language_folder: ClassVar[Path] = _LANGUAGE_DIR

    def __init__(
        self,
        code: ui.TMaybeRef[str],
        *,
        language: Optional[ui.TMaybeRef[str]] = None,
        theme: Optional[ui.TMaybeRef[str]] = None,
        themes: Optional[Dict[str, str]] = None,
        transformers: Optional[List[TTransformerNames]] = None,
        line_numbers: Optional[ui.TMaybeRef[bool]] = None,
    ):
        super().__init__()
        self.props({"code": code})

        if language:
            self.props({"language": language})

        if theme:
            self.props({"theme": theme})

        if themes:
            self.props({"themes": themes})

        if transformers:
            self.props({"transformers": transformers})

        if line_numbers is not None:
            self.props({"lineNumbers": line_numbers})

    def _to_json_dict(self):
        self.use_zero_dependency()
        return super()._to_json_dict()

    def use_zero_dependency(self):
        app = get_app_slot()
        tag_name = self.dependency.tag_name  # type: ignore

        if app.mode != "zero" or app.has_temp_component_dependency(tag_name):
            return

        self.update_dependencies(
            css=[_STYLE], externals=_ZERO_IMPORT_MAPS, replace=True
        )

    @staticmethod
    def update_zero_dependency(add_languages: Optional[Iterable[str]] = None):
        if isinstance(add_languages, str):
            add_languages = [add_languages]

        for lang in add_languages or []:
            name = f"{_LANGS_IMPORT_NAME}{lang}.mjs"
            path = _LANG_DIR / f"{lang}.mjs"
            _ZERO_IMPORT_MAPS[name] = path


TTransformerNames = Literal["notationDiff"]



from typing import Any, Dict, Literal


TLocale = Literal[
    "zh-CN",
    "en-US",
    "ja-JP",
    "zh-TW",
    "pt-PT",
    "es-ES",
    "id-ID",
    "fr-FR",
    "de-DE",
    "ko-KR",
    "it-IT",
    "ms-MY",
    "th-TH",
    "vi-VN",
    "km-KH",
    "ar-EG",
    "ru-RU",
]

TCustomizeLocale = Dict[str, Any]



import importlib
from typing import Dict
from instaui.arco.types import TLocale


class LazyLocaleDict:
    def __init__(self):
        self._locale_map = {}

    def get(self, key: str):
        key = key.lower().replace("-", "_")

        if key not in self._locale_map:
            self._locale_map[key] = self._load_locale(key)
        return self._locale_map[key]

    def _load_locale(self, key: str):
        module_name = key
        try:
            module = importlib.import_module(f"instaui.arco.locales.{module_name}")
            return module.locale

        except ImportError:
            raise ImportError(f"Cannot import module {module_name}")


_locale_dict = LazyLocaleDict()


def get_locale(locale: TLocale) -> Dict:
    return _locale_dict.get(locale)



locale = {
    "locale": "zh-CN",
    "empty": {"description": "暂无数据"},
    "drawer": {"okText": "确定", "cancelText": "取消"},
    "popconfirm": {"okText": "确定", "cancelText": "取消"},
    "modal": {"okText": "确定", "cancelText": "取消"},
    "pagination": {
        "goto": "前往",
        "page": "页",
        "countPerPage": "条/页",
        "total": "共 {0} 条",
    },
    "table": {"okText": "确定", "resetText": "重置"},
    "upload": {
        "start": "开始",
        "cancel": "取消",
        "delete": "删除",
        "retry": "点击重试",
        "buttonText": "点击上传",
        "preview": "预览",
        "drag": "点击或拖拽文件到此处上传",
        "dragHover": "释放文件并开始上传",
        "error": "上传失败",
    },
    "calendar": {
        "formatYear": "YYYY 年",
        "formatMonth": "YYYY 年 MM 月",
        "today": "今天",
        "view": {"month": "月", "year": "年", "week": "周", "day": "日"},
        "month": {
            "long": {
                "January": "一月",
                "February": "二月",
                "March": "三月",
                "April": "四月",
                "May": "五月",
                "June": "六月",
                "July": "七月",
                "August": "八月",
                "September": "九月",
                "October": "十月",
                "November": "十一月",
                "December": "十二月",
            },
            "short": {
                "January": "一月",
                "February": "二月",
                "March": "三月",
                "April": "四月",
                "May": "五月",
                "June": "六月",
                "July": "七月",
                "August": "八月",
                "September": "九月",
                "October": "十月",
                "November": "十一月",
                "December": "十二月",
            },
        },
        "week": {
            "long": {
                "self": "周",
                "monday": "周一",
                "tuesday": "周二",
                "wednesday": "周三",
                "thursday": "周四",
                "friday": "周五",
                "saturday": "周六",
                "sunday": "周日",
            },
            "short": {
                "self": "周",
                "monday": "一",
                "tuesday": "二",
                "wednesday": "三",
                "thursday": "四",
                "friday": "五",
                "saturday": "六",
                "sunday": "日",
            },
        },
    },
    "datePicker": {
        "view": {"month": "月", "year": "年", "week": "周", "day": "日"},
        "month": {
            "long": {
                "January": "一月",
                "February": "二月",
                "March": "三月",
                "April": "四月",
                "May": "五月",
                "June": "六月",
                "July": "七月",
                "August": "八月",
                "September": "九月",
                "October": "十月",
                "November": "十一月",
                "December": "十二月",
            },
            "short": {
                "January": "一月",
                "February": "二月",
                "March": "三月",
                "April": "四月",
                "May": "五月",
                "June": "六月",
                "July": "七月",
                "August": "八月",
                "September": "九月",
                "October": "十月",
                "November": "十一月",
                "December": "十二月",
            },
        },
        "week": {
            "long": {
                "self": "周",
                "monday": "周一",
                "tuesday": "周二",
                "wednesday": "周三",
                "thursday": "周四",
                "friday": "周五",
                "saturday": "周六",
                "sunday": "周日",
            },
            "short": {
                "self": "周",
                "monday": "一",
                "tuesday": "二",
                "wednesday": "三",
                "thursday": "四",
                "friday": "五",
                "saturday": "六",
                "sunday": "日",
            },
        },
        "placeholder": {
            "date": "请选择日期",
            "week": "请选择周",
            "month": "请选择月份",
            "year": "请选择年份",
            "quarter": "请选择季度",
            "time": "请选择时间",
        },
        "rangePlaceholder": {
            "date": ["开始日期", "结束日期"],
            "week": ["开始周", "结束周"],
            "month": ["开始月份", "结束月份"],
            "year": ["开始年份", "结束年份"],
            "quarter": ["开始季度", "结束季度"],
            "time": ["开始时间", "结束时间"],
        },
        "selectTime": "选择时间",
        "today": "今天",
        "now": "此刻",
        "ok": "确定",
    },
    "image": {"loading": "加载中"},
    "imagePreview": {
        "fullScreen": "全屏",
        "rotateRight": "向右旋转",
        "rotateLeft": "向左旋转",
        "zoomIn": "放大",
        "zoomOut": "缩小",
        "originalSize": "原始尺寸",
    },
    "typography": {
        "copied": "已复制",
        "copy": "复制",
        "expand": "展开",
        "collapse": "折叠",
        "edit": "编辑",
    },
    "form": {
        "validateMessages": {
            "required": "#{field} 是必填项",
            "type": {
                "string": "#{field} 不是合法的文本类型",
                "number": "#{field} 不是合法的数字类型",
                "boolean": "#{field} 不是合法的布尔类型",
                "array": "#{field} 不是合法的数组类型",
                "object": "#{field} 不是合法的对象类型",
                "url": "#{field} 不是合法的 url 地址",
                "email": "#{field} 不是合法的邮箱地址",
                "ip": "#{field} 不是合法的 IP 地址",
            },
            "number": {
                "min": "`#{value}` 小于最小值 `#{min}`",
                "max": "`#{value}` 大于最大值 `#{max}`",
                "equal": "`#{value}` 不等于 `#{equal}`",
                "range": "`#{value}` 不在 `#{min} ~ #{max}` 范围内",
                "positive": "`#{value}` 不是正数",
                "negative": "`#{value}` 不是负数",
            },
            "array": {
                "length": "`#{field}` 个数不等于 #{length}",
                "minLength": "`#{field}` 个数最少为 #{minLength}",
                "maxLength": "`#{field}` 个数最多为 #{maxLength}",
                "includes": "#{field} 不包含 #{includes}",
                "deepEqual": "#{field} 不等于 #{deepEqual}",
                "empty": "`#{field}` 不是空数组",
            },
            "string": {
                "minLength": "字符数最少为 #{minLength}",
                "maxLength": "字符数最多为 #{maxLength}",
                "length": "字符数必须是 #{length}",
                "match": "`#{value}` 不符合模式 #{pattern}",
                "uppercase": "`#{value}` 必须全大写",
                "lowercase": "`#{value}` 必须全小写",
            },
            "object": {
                "deepEqual": "`#{field}` 不等于期望值",
                "hasKeys": "`#{field}` 不包含必须字段",
                "empty": "`#{field}` 不是对象",
            },
            "boolean": {"true": "期望是 `true`", "false": "期望是 `false`"},
        }
    },
    "colorPicker": {
        "history": "最近使用颜色",
        "preset": "系统预设颜色",
        "empty": "暂无",
    },
}



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Tree(Element):
    def __init__(
        self,
        **kwargs: Unpack[component_types.TTree],
    ):
        super().__init__("a-tree")

        self.props(handle_props(kwargs))  # type: ignore

    def on_select(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "select",
            handler,
            extends=extends,
        )
        return self

    def on_check(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "check",
            handler,
            extends=extends,
        )
        return self

    def on_expand(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "expand",
            handler,
            extends=extends,
        )
        return self

    def on_drag_start(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "drag-start",
            handler,
            extends=extends,
        )
        return self

    def on_drag_end(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "drag-end",
            handler,
            extends=extends,
        )
        return self

    def on_drag_over(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "drag-over",
            handler,
            extends=extends,
        )
        return self

    def on_drag_leave(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "drag-leave",
            handler,
            extends=extends,
        )
        return self

    def on_drop(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "drop",
            handler,
            extends=extends,
        )
        return self



from typing import get_args
from instaui import ui
from instaui.arco.types import TLocale
from instaui.arco.locales import get_locale


class Locale:
    def __init__(self, locale: TLocale):
        self._names = get_args(TLocale)
        self._current_name = ui.state(locale)

        @ui.computed(inputs=[self._current_name])
        def locale_data(name: TLocale):
            return get_locale(name)

        self._current_data = locale_data

    @property
    def all_names(self):
        return self._names

    @property
    def current_name(self):
        return self._current_name

    @property
    def current(self):
        return self._current_data


def use_locale(locale_name: TLocale):
    """Reactive language settings.

    Args:
        locale_name (TLocale):  The language name of the locale.

    Example:
    .. code-block:: python
        @ui.page("/")
        def index():
            locale = arco.use_locale("zh-CN")
            arco.select(locale.all_names, locale.current_name)

            with arco.config_provider(locale=locale.current):
                arco.pagination(
                    total=50, show_total=True, show_jumper=True, show_page_size=True
                )

    """
    return Locale(locale_name)



import typing
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from .input import Input


class TInputSearch(component_types.TInput):
    search_button: bool
    loading: bool
    disabled: bool
    size: typing.Literal["mini", "small", "medium", "large"]
    button_text: str
    button_props: typing.Dict[str, typing.Any]


class InputSearch(Input, exten_name="search"):
    def __init__(
        self,
        value: typing.Optional[TMaybeRef[str]] = None,
        **kwargs: Unpack[TInputSearch],
    ):
        super().__init__(value=value, **kwargs)

    def on_search(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "search",
            handler,
            extends=extends,
        )
        return self



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props
from .tab_pane import TabPane


class Tabs(Element):
    def __init__(
        self,
        active_key: typing.Optional[TMaybeRef[typing.Union[str, int]]] = None,
        *,
        active_key_model_value: typing.Optional[typing.Union[str, int]] = None,
        **kwargs: Unpack[component_types.TTabs],
    ):
        """Tabs element.

        Example:
        .. code-block:: python
            with arco.tabs() as tabs:
                with tabs.add_pane(key="1", title="Tab 1"):
                    html.paragraph("tab 1")
                with tabs.add_pane(key="2", title="Tab 2"):
                    html.paragraph("tab 2")
                with tabs.add_pane(key="3", title="Tab 3"):
                    html.paragraph("tab 3")

        """
        super().__init__("a-tabs")
        self.props(handle_props(kwargs))  # type: ignore

        if active_key is not None:
            self.vmodel(active_key, prop_name="active-key")

        if active_key_model_value is not None:
            self.props({"active-key": active_key_model_value})

    def add_pane(self, *, key: str, title: str):
        """Add a new pane to the tabs.

        Args:
            key (str): key of the pane.
            title (str): title of the pane.

        """
        return TabPane(key=key, title=title)

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self

    def on_tab_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "tab-click",
            handler,
            extends=extends,
        )
        return self

    def on_add(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "add",
            handler,
            extends=extends,
        )
        return self

    def on_delete(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "delete",
            handler,
            extends=extends,
        )
        return self



from __future__ import annotations
import typing
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui.components.element import Element
from instaui.components.content import Content
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Typography(Element):
    _exten_name: typing.ClassVar[typing.Optional[str]] = None

    def __init__(
        self,
        text: typing.Optional[TMaybeRef[str]] = None,
        **kwargs: Unpack[component_types.TTypography],
    ):
        tag = f"a-typography{'-' + self._exten_name if self._exten_name else ''}"
        super().__init__(tag)

        if text is not None:
            with self:
                Content(text)

        self.props(handle_props(kwargs))  # type: ignore

    def __init_subclass__(cls, *, exten_name: str = "") -> None:
        cls._exten_name = exten_name

    def on_edit_start(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "edit-start",
            handler,
            extends=extends,
        )
        return self

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self

    def on_edit_end(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "edit-end",
            handler,
            extends=extends,
        )
        return self

    def on_copy(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "copy",
            handler,
            extends=extends,
        )
        return self

    def on_ellipsis(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "ellipsis",
            handler,
            extends=extends,
        )
        return self

    def on_expand(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "expand",
            handler,
            extends=extends,
        )
        return self


class TypographyTitle(Typography, exten_name="title"):
    def __init__(
        self,
        text: typing.Optional[TMaybeRef[str]] = None,
        *,
        heading: typing.Optional[
            TMaybeRef[typing.Literal["1", "2", "3", "4", "5", "6"]]
        ] = None,
        **kwargs: Unpack[component_types.TTypography],
    ):
        super().__init__(text=text, **kwargs)

        if heading is not None:
            self.props({"heading": heading})


class TypographyParagraph(Typography, exten_name="paragraph"):
    def __init__(
        self,
        text: typing.Optional[TMaybeRef[str]] = None,
        *,
        blockquote: typing.Optional[TMaybeRef[bool]] = None,
        spacing: typing.Optional[TMaybeRef[typing.Literal["default", "close"]]] = None,
        **kwargs: Unpack[component_types.TTypography],
    ):
        super().__init__(text=text, **kwargs)

        if blockquote is not None:
            self.props({"blockquote": blockquote})

        if spacing is not None:
            self.props({"spacing": spacing})



from __future__ import annotations
import typing
from instaui import ui
from instaui.vars.types import TMaybeRef
from instaui.components.element import Element
from instaui.vars.mixin_types.element_binding import ElementBindingMixin


class Icon(Element):
    def __init__(self, name: typing.Optional[TMaybeRef[str]] = None):
        if isinstance(name, ElementBindingMixin):
            name = ui.str_format(r"a-icon-{}", name)

        else:
            name = f"a-icon-{name}"

        super().__init__(name)



import typing
from typing_extensions import Unpack
from instaui.vars.types import TMaybeRef
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from .input import Input
from ._utils import try_setup_vmodel


class TInputPassword(component_types.TInput, total=False):
    model_visibility: bool
    default_visibility: bool
    invisible_button: bool


class InputPassword(Input, exten_name="password"):
    def __init__(
        self,
        value: typing.Optional[TMaybeRef[str]] = None,
        *,
        visibility_value: typing.Optional[TMaybeRef[bool]] = None,
        **kwargs: Unpack[TInputPassword],
    ):
        super().__init__(value=value, **kwargs)
        try_setup_vmodel(self, visibility_value, prop_name="visibility")

    def on_visibility_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "visibility-change",
            handler,
            extends=extends,
        )
        return self



import typing
from instaui.vars.types import TMaybeRef
from instaui.components.element import Element


class TabPane(Element):
    def __init__(self, *, key: str, title: typing.Optional[TMaybeRef[str]] = None):
        super().__init__("a-tab-pane")

        self.key(key)
        if title is not None:
            self.props({"title": title})



"""
Easy to use arco.design for InstaUI.

Examples:
.. code-block:: python
    from instaui import ui, arco

    arco.use()

    @ui.page("/")
    def index_page():
        arco.input(placeholder="input")
"""

__all__ = [
    "use",
    "layout_header",
    "layout_footer",
    "layout_content",
    "layout_sider",
    "button",
    "icon",
    "link",
    "typography",
    "typography_title",
    "typography_paragraph",
    "divider",
    "layout",
    "space",
    "avatar",
    "badge",
    "calendar",
    "card",
    "carousel",
    "collapse",
    "comment",
    "descriptions",
    "empty",
    "image",
    "list",
    "popover",
    "statistic",
    "table",
    "tabs",
    "tag",
    "timeline",
    "tooltip",
    "tree",
    "auto_complete",
    "cascader",
    "checkbox",
    "color_picker",
    "date_picker",
    "form",
    "input",
    "input_password",
    "input_search",
    "input_number",
    "verification_code",
    "input_tag",
    "mention",
    "radio",
    "rate",
    "select",
    "slider",
    "switch",
    "textarea",
    "time_picker",
    "transfer",
    "tree_select",
    "upload",
    "alert",
    "drawer",
    "modal",
    "pop_confirm",
    "progress",
    "result",
    "spin",
    "skeleton",
    "breadcrumb",
    "dropdown",
    "menu",
    "menu_item",
    "sub_menu",
    "page_header",
    "pagination",
    "steps",
    "affix",
    "anchor",
    "back_top",
    "resize_box",
    "trigger",
    "split",
    "overflow_list",
    "watermark",
    "radio_group",
    "tab_pane",
    "config_provider",
    "use_locale",
]


from .setup import use
from .components.button import Button as button
from .components.icon import Icon as icon
from .components.link import Link as link
from .components.typography import (
    Typography as typography,
    TypographyTitle as typography_title,
    TypographyParagraph as typography_paragraph,
)
from .components.divider import Divider as divider
from .components.layout import Layout as layout
from .components.space import Space as space
from .components.avatar import Avatar as avatar
from .components.badge import Badge as badge
from .components.calendar import Calendar as calendar
from .components.card import Card as card
from .components.carousel import Carousel as carousel
from .components.collapse import Collapse as collapse
from .components.comment import Comment as comment
from .components.descriptions import Descriptions as descriptions
from .components.empty import Empty as empty
from .components.image import Image as image
from .components.list import List as list
from .components.popover import Popover as popover
from .components.statistic import Statistic as statistic
from .components.table import Table as table
from .components.tabs import Tabs as tabs
from .components.tag import Tag as tag
from .components.timeline import Timeline as timeline
from .components.tooltip import Tooltip as tooltip
from .components.tree import Tree as tree
from .components.auto_complete import AutoComplete as auto_complete
from .components.cascader import Cascader as cascader
from .components.checkbox import Checkbox as checkbox
from .components.color_picker import ColorPicker as color_picker
from .components.date_picker import DatePicker as date_picker
from .components.form import Form as form
from .components.input import Input as input
from .components.input_password import InputPassword as input_password
from .components.input_search import InputSearch as input_search
from .components.input_number import InputNumber as input_number
from .components.verification_code import VerificationCode as verification_code
from .components.input_tag import InputTag as input_tag
from .components.mention import Mention as mention
from .components.radio import Radio as radio
from .components.rate import Rate as rate
from .components.select import Select as select
from .components.slider import Slider as slider
from .components.switch import Switch as switch
from .components.textarea import Textarea as textarea
from .components.time_picker import TimePicker as time_picker
from .components.transfer import Transfer as transfer
from .components.tree_select import TreeSelect as tree_select
from .components.upload import Upload as upload
from .components.alert import Alert as alert
from .components.drawer import Drawer as drawer
from .components.modal import Modal as modal
from .components.pop_confirm import PopConfirm as pop_confirm
from .components.progress import Progress as progress
from .components.result import Result as result
from .components.spin import Spin as spin
from .components.skeleton import Skeleton as skeleton
from .components.breadcrumb import Breadcrumb as breadcrumb
from .components.dropdown import Dropdown as dropdown
from .components.menu import (
    Menu as menu,
    MenuItem as menu_item,
    SubMenu as sub_menu,
)
from .components.page_header import PageHeader as page_header
from .components.pagination import Pagination as pagination
from .components.steps import Steps as steps
from .components.affix import Affix as affix
from .components.anchor import Anchor as anchor
from .components.back_top import BackTop as back_top
from .components.config_provider import ConfigProvider as config_provider
from .components.resize_box import ResizeBox as resize_box
from .components.trigger import Trigger as trigger
from .components.split import Split as split
from .components.overflow_list import OverflowList as overflow_list
from .components.watermark import Watermark as watermark

from .components.layout_header import LayoutHeader as layout_header
from .components.layout_footer import LayoutFooter as layout_footer
from .components.layout_content import LayoutContent as layout_content
from .components.layout_sider import LayoutSider as layout_sider
from .components.radio_group import RadioGroup as radio_group
from .components.tab_pane import TabPane as tab_pane
from ._use_tools.locale import use_locale



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props, try_setup_vmodel

_TResizeBoxSizeValue = typing.Union[int, float]


class ResizeBox(Element):
    def __init__(
        self,
        width_value: typing.Optional[TMaybeRef[_TResizeBoxSizeValue]] = None,
        height_value: typing.Optional[TMaybeRef[_TResizeBoxSizeValue]] = None,
        **kwargs: Unpack[component_types.TResizebox],
    ):
        super().__init__("a-resize-box")

        try_setup_vmodel(self, width_value, prop_name="width")
        try_setup_vmodel(self, height_value, prop_name="height")
        self.props(handle_props(kwargs))  # type: ignore

    def on_moving_start(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "moving-start",
            handler,
            extends=extends,
        )
        return self

    def on_moving(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "moving",
            handler,
            extends=extends,
        )
        return self

    def on_moving_end(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "moving-end",
            handler,
            extends=extends,
        )
        return self



from __future__ import annotations
import typing
from typing_extensions import Unpack
import pydantic
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.vars.state import StateModel
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props
from .radio_group import RadioGroup

_TValue = typing.Union[str, float]


class Radio(Element):
    def __init__(
        self,
        value: typing.Optional[TMaybeRef[_TValue]] = None,
        **kwargs: Unpack[component_types.TRadio],
    ):
        super().__init__("a-radio")

        self.props({"value": value})
        self.props(handle_props(kwargs))  # type: ignore

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self

    @staticmethod
    def from_list(options: TMaybeRef[typing.List[_TValue]], value: TMaybeRef[_TValue]):
        return RadioGroup(value=value, options=options)  # type: ignore

    @staticmethod
    def from_options(
        options: TMaybeRef[typing.List[RadioOption]], value: TMaybeRef[_TValue]
    ):
        return RadioGroup(value=value, options=options)  # type: ignore

    class RadioOption(StateModel):
        label: str
        value: typing.Union[str, int]
        disabled: bool = pydantic.Field(default=False)



import typing
from typing_extensions import Unpack
from instaui.components.element import Element
from instaui.arco import component_types
from instaui.event.event_mixin import EventMixin
from ._utils import handle_props


class Table(Element):
    def __init__(
        self,
        **kwargs: Unpack[component_types.TTable],
    ):
        super().__init__("a-table")

        self.props(handle_props(kwargs))  # type: ignore

    def on_expand(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "expand",
            handler,
            extends=extends,
        )
        return self

    def on_expanded_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "expanded-change",
            handler,
            extends=extends,
        )
        return self

    def on_select(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "select",
            handler,
            extends=extends,
        )
        return self

    def on_select_all(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "select-all",
            handler,
            extends=extends,
        )
        return self

    def on_selection_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "selection-change",
            handler,
            extends=extends,
        )
        return self

    def on_sorter_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "sorter-change",
            handler,
            extends=extends,
        )
        return self

    def on_filter_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "filter-change",
            handler,
            extends=extends,
        )
        return self

    def on_page_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "page-change",
            handler,
            extends=extends,
        )
        return self

    def on_page_size_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "page-size-change",
            handler,
            extends=extends,
        )
        return self

    def on_change(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "change",
            handler,
            extends=extends,
        )
        return self

    def on_cell_mouse_enter(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "cell-mouse-enter",
            handler,
            extends=extends,
        )
        return self

    def on_cell_mouse_leave(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "cell-mouse-leave",
            handler,
            extends=extends,
        )
        return self

    def on_cell_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "cell-click",
            handler,
            extends=extends,
        )
        return self

    def on_row_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "row-click",
            handler,
            extends=extends,
        )
        return self

    def on_header_click(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "header-click",
            handler,
            extends=extends,
        )
        return self

    def on_column_resize(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "column-resize",
            handler,
            extends=extends,
        )
        return self

    def on_row_dblclick(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "row-dblclick",
            handler,
            extends=extends,
        )
        return self

    def on_cell_dblclick(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "cell-dblclick",
            handler,
            extends=extends,
        )
        return self

    def on_row_contextmenu(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "row-contextmenu",
            handler,
            extends=extends,
        )
        return self

    def on_cell_contextmenu(
        self,
        handler: EventMixin,
        *,
        extends: typing.Optional[typing.List] = None,
    ):
        self.on(
            "cell-contextmenu",
            handler,
            extends=extends,
        )
        return self
